@@ -1,6 +1,7 @@
 /* Startup code for Insight.
-   Copyright 1994, 1995, 1996, 1997, 1998, 2000, 200, 2002
-   Free Software Foundation, Inc.
+
+   Copyright 1994, 1995, 1996, 1997, 1998, 2000, 200, 2002, 2003 Free
+   Software Foundation, Inc.
 
    Written by Stu Grossman <grossman@cygnus.com> of Cygnus Support.
 
@@ -31,6 +32,7 @@
 #include "gdb-events.h"
 #include "top.h"
 #include "annotate.h"
+#include "cli/cli-decode.h"
 
 #ifdef _WIN32
 #define WIN32_LEAN_AND_MEAN
@@ -42,7 +44,8 @@
 #define HAS_STDARG 1
 
 #include <itcl.h>
-#include <tix.h>
+#include <tcl.h>
+#include <tk.h>
 #include "guitcl.h"
 #include "gdbtk.h"
 
@@ -82,7 +85,6 @@ static void gdbtk_attach (void);
 static void gdbtk_detach (void);
 static void gdbtk_file_changed (char *);
 static void gdbtk_exec_file_display (char *);
-static void tk_command_loop (void);
 static void gdbtk_call_command (struct cmd_list_element *, char *, int);
 static ptid_t gdbtk_wait (ptid_t, struct target_waitstatus *);
 int x_event (int);
@@ -132,7 +134,6 @@ gdbtk_add_hooks (void)
   set_gdb_event_hooks (&handlers);
 
   /* Hooks */
-  command_loop_hook = tk_command_loop;
   call_command_hook = gdbtk_call_command;
   set_hook = gdbtk_set_hook;
   readline_begin_hook = gdbtk_readline_begin;
@@ -167,6 +168,7 @@ gdbtk_add_hooks (void)
   error_begin_hook = gdbtk_error_begin;
 
   annotate_signal_hook = gdbtk_annotate_signal;
+  annotate_signalled_hook = gdbtk_annotate_signal;
 }
 
 /* These control where to put the gdb output which is created by
@@ -357,40 +359,6 @@ gdbtk_memory_changed (CORE_ADDR addr, int len)
 }
 
 
-/* This function is called instead of gdb's internal command loop.  This is the
-   last chance to do anything before entering the main Tk event loop. 
-   At the end of the command, we enter the main loop. */
-
-static void
-tk_command_loop ()
-{
-  extern FILE *instream;
-
-  /* We no longer want to use stdin as the command input stream */
-  instream = NULL;
-
-  if (Tcl_Eval (gdbtk_interp, "gdbtk_tcl_preloop") != TCL_OK)
-    {
-      char *msg;
-
-      /* Force errorInfo to be set up propertly.  */
-      Tcl_AddErrorInfo (gdbtk_interp, "");
-
-      msg = Tcl_GetVar (gdbtk_interp, "errorInfo", TCL_GLOBAL_ONLY);
-#ifdef _WIN32
-      MessageBox (NULL, msg, NULL, MB_OK | MB_ICONERROR | MB_TASKMODAL);
-#else
-      fputs_unfiltered (msg, gdb_stderr);
-#endif
-    }
-
-#ifdef _WIN32
-  close_bfds ();
-#endif
-
-  Tk_MainLoop ();
-}
-
 /* This hook is installed as the ui_loop_hook, which is used in several
  * places to keep the gui alive (x_event runs gdbtk's event loop). Users
  * include:
@@ -517,6 +485,9 @@ static void
 gdbtk_call_command (struct cmd_list_element *cmdblk,
 		    char *arg, int from_tty)
 {
+  struct cleanup *old_chain;
+
+  old_chain = make_cleanup (null_cleanup, 0);
   running_now = 0;
   if (cmdblk->class == class_run || cmdblk->class == class_trace)
     {
@@ -524,13 +495,15 @@ gdbtk_call_command (struct cmd_list_element *cmdblk,
       running_now = 1;
       if (!No_Update)
 	Tcl_Eval (gdbtk_interp, "gdbtk_tcl_busy");
-      (*cmdblk->function.cfunc) (arg, from_tty);
+      cmd_func (cmdblk, arg, from_tty);
       running_now = 0;
       if (!No_Update)
 	Tcl_Eval (gdbtk_interp, "gdbtk_tcl_idle");
     }
   else
-    (*cmdblk->function.cfunc) (arg, from_tty);
+    cmd_func (cmdblk, arg, from_tty);
+
+  do_cleanups (old_chain);
 }
 
 /* Called after a `set' command succeeds.  Runs the Tcl hook
@@ -735,7 +708,15 @@ gdbtk_trace_start_stop (int start, int from_tty)
 static void
 gdbtk_selected_frame_changed (int level)
 {
-  Tcl_UpdateLinkedVar (gdbtk_interp, "gdb_selected_frame_level");
+#if TCL_MAJOR_VERSION == 8 && TCL_MINOR_VERSION < 1
+  char *a;
+  xasprintf (&a, "%d", level);
+  Tcl_SetVar (gdbtk_interp, "gdb_selected_frame_level", a, TCL_GLOBAL_ONLY);
+  xfree (a);
+#else
+  Tcl_SetVar2Ex (gdbtk_interp, "gdb_selected_frame_level", NULL,
+		 Tcl_NewIntObj (level), TCL_GLOBAL_ONLY);
+#endif
 }
 
 /* Called when the current thread changes. */
